#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstring>
#include <cassert>
#include <cstdarg>

using namespace std;

const int EXIT_AC = 42;
const int EXIT_WA = 43;

std::ifstream input, judgeans;
FILE * judgemessage = NULL;

void wrong_answer (const char * err, ...)
{
  va_list pvar;
  va_start(pvar, err);
  fprintf(judgemessage, "Wrong answer\n");
  vfprintf(judgemessage, err, pvar);
  fprintf(judgemessage, "\n");
  exit(EXIT_WA);
}

void judge_error (const char * err, ...)
{
  va_list pvar;
  va_start(pvar, err);
  // If judgemessage hasn't been set up yet, write error to stderr
  if (!judgemessage) judgemessage = stderr;
  vfprintf(judgemessage, err, pvar);
  fprintf(judgemessage, "\n");
  assert(!"Judge Error");
}

template <typename Stream>
void openfile (Stream & stream, const char * file, const char * whoami) 
{
  stream.open(file);
  if (stream.fail()) judge_error("%s: failed to open %s\n", whoami, file);
}

FILE * openfeedback (const char * feedbackdir, const char * feedback, const char * whoami) 
{
  std::string path = std::string(feedbackdir) + "/" + std::string(feedback);
  FILE * res = fopen(path.c_str(), "w");
  if (!res) judge_error("%s: failed to open %s for writing", whoami, path.c_str());
  return res;
}

const char * USAGE = "Usage: %s judge_in judge_ans feedback_dir [options] < team_out";

#include <bits/stdc++.h>
using namespace std;

typedef long long int ll;
typedef pair<int, int> pii;
bool R(){return true;}
template<typename T, typename ... Args>
bool R(T & x, Args & ... args){return cin >> x && R(args...);}
void init_io(){ios::sync_with_stdio(false);cin.tie(0);}

#define PB push_back
#define FOR(prom, a, b) for(int prom = (a); prom < (b); prom++)
#define FORDE(prom, a, b) for(int prom = (a); prom >= (b); prom--)
#define ALL(vec) (vec).begin(), (vec).end()
#define MM(co, cim) memset((co), (cim), sizeof((co)))
#define DEB(x) cerr << ">>> " << #x << " : " << x << endl;
#define INF   1000000007
#define INFLL 4000000000000000007LL

#define MAX_SIZE 3000

map<int, char> chars = {{0, '#'}, {1, '.'}};
map<int, char> check_chars = {{0, '1'}, {1, '2'}, {2, '3'}, {3, '4'}};
set<char> valid_chars = {'#', '.'};

//input
int n, m;
char grid[MAX_SIZE + 14][MAX_SIZE + 14];
string s;

string parse (char (&arr)[MAX_SIZE + 14][MAX_SIZE + 14], int si, int sj, int chari, int outcharj, bool validate)
{
  set<pii> togo;
  queue<pii> q;
  q.push({si, sj});
  arr[si][sj] = check_chars[outcharj];
  while (!q.empty())
  {
    pii cc = q.front();
    q.pop();
    int ci = cc.first;
    int cj = cc.second;
    FOR(di, -1, 2) FOR(dj, -1, 2) if (di * di + dj * dj == 1)
    {
       int ni = ci + di;
       int nj = cj + dj;
       if (ni >= 0 && nj >= 0 && ni <= MAX_SIZE + 1 && nj <= MAX_SIZE + 1 && arr[ni][nj] != '$' && (arr[ni][nj] < '1' || arr[ni][nj] > '4'))
       {
         if (arr[ni][nj] == chars[chari]) 
         {
           arr[ni][nj] = check_chars[outcharj];
           q.push({ni, nj});
         }
         else togo.insert({ni, nj});
       }
    }
  }
  string res;
  int total = 0;
  for (auto en : togo) if (arr[en.first][en.second] != '$' && (arr[en.first][en.second] < '1' || arr[en.first][en.second] > '4'))
  {
    res += parse(arr, en.first, en.second, 1 - chari, (outcharj + 1) % 4, validate);
    ++total;
  }
  if (validate)
  {
    if (total > 26) wrong_answer("Blob contains more than 26 sub-blobs");
  }
  if (total) res = ((char)(total + 'a' - 1)) + res;
  return res;
}

void read_input (void)
{
  if (!(input >> n >> m)) judge_error("Could not read input");
  
  FOR(i, 0, n + 2) FOR(j, 0, m + 2) grid[i][j] = '$';
  
  string line;
  FOR(i, 1, n + 1)
  {
    if (!(input >> line)) judge_error("Could not read input");
    FOR(j, 1, m + 1) grid[i][j] = line[j - 1];
  }
  s = parse(grid, 1, 1, 0, 0, false);
}

int stud_n, stud_m;
string stud_s;
char stud_grid[MAX_SIZE + 14][MAX_SIZE + 14];

void check_answer (void)
{
  //read first part of the answer
  if (!(cin >> stud_n >> stud_m)) wrong_answer("Could not read dimensions of grid");
  if (stud_n < 1 || stud_n > MAX_SIZE) wrong_answer("Too small/large row dimension (N=%d)", stud_n);
  if (stud_m < 1 || stud_m > MAX_SIZE) wrong_answer("Too small/large col dimension (M=%d)", stud_m);
  
  FOR(i, 0, stud_n + 2) FOR(j, 0, stud_m + 2) stud_grid[i][j] = '.';
  FOR(i, 0, stud_n + 3) stud_grid[i][stud_m + 2] = '$';
  FOR(j, 0, stud_m + 3) stud_grid[stud_n + 2][j] = '$';
  
  string line;
  FOR(i, 1, stud_n + 1)
  {
    if (!(cin >> line)) wrong_answer("Could not read %dth line of solution", i);
    if ((int)line.length() != stud_m) wrong_answer("%dth line of solution is %d of %d reported columns long", line.length(), stud_m);
    FOR(j, 1, stud_m + 1)
    {
      char c = line[j - 1];
      if (!valid_chars.count(c)) wrong_answer("Character %c on (%d,%d) coordinates in grid is not valid", c, i, j);
      stud_grid[i][j] = c;
    }
  }
  stud_s = parse(stud_grid, 0, 0, 1, 0, true);
  if (!stud_s.length()) wrong_answer("There is no top level #blob");
  if (stud_s[0] != 'a') wrong_answer("There is more than one top level #blob");
  //check validity of the input - border
  for (int i = 1; i < stud_n + 1; i++)
  {
    for (int j = 1; j < stud_m + 1; j++)
    {
      FOR(di, -1, 2) FOR(dj, -1, 2) if (di || dj) 
      {
        int curr = stud_grid[i][j] - '0';
        int other = stud_grid[i + di][j + dj] - '0';
        if (!(curr == other || ((curr % 2) != (other % 2)))) wrong_answer("Borders are ill formed");
      }
    }
  }
  
  stud_s = stud_s.substr(1);
  reverse(ALL(stud_s));
  
  if (stud_s != s)
  {
    reverse(ALL(stud_s));
    wrong_answer("Output %s is not reversed %s", stud_s.c_str(), s.c_str());
  }
}

int main (int argc, char * argv[])
{
  init_io();
  judgemessage = openfeedback(argv[3], "judgemessage.txt", argv[0]);
  
  if (argc < 4) judge_error(USAGE, argv[0]);
	
  openfile(input, argv[1], argv[0]);
  openfile(judgeans, argv[2], argv[0]);

  //read input
  read_input();
  
  //check team result (e.g. YES/NO, or size of solution on a single line)
  //check_result();
  //check answer constructed by team
  check_answer();
  
  char c;
  if (cin >> c) wrong_answer("Garbage at the end of file");
	
  exit(EXIT_AC);
}
